from tkinter import *
import random

def hex_to_rgb(value):
    """Return (red, green, blue) for the color given as #rrggbb."""
    value = value.lstrip('#')
    lv = len(value)
    return tuple(int(value[i:i + lv // 3], 16) for i in range(0, lv, lv // 3))

def rgb_to_hex(red, green, blue):
    """Return color as #rrggbb for the given color values."""
    return '#%02x%02x%02x' % (red, green, blue)

"""
hex_to_rgb("#ffffff")           #==> (255, 255, 255)
hex_to_rgb("#ffffffffffff")     #==> (65535, 65535, 65535)
rgb_to_hex(255, 255, 255)       #==> '#ffffff'
rgb_to_hex(65535, 65535, 65535) #==> '#ffffffffffff'
"""

def draw_tree(tree_list, pos, side_len):
    if isinstance(tree_list[0], int):
        col=rgb_to_hex(tree_list[0], tree_list[0], tree_list[0])
        canvas.create_rectangle(pos[0], pos[1], pos[0]+side_len, pos[1]+side_len, outline=col, fill=col)
    else:
        draw_tree(tree_list[0], pos, side_len / 2)
    if isinstance(tree_list[1], int):
        col=rgb_to_hex(tree_list[1], tree_list[1], tree_list[1])
        canvas.create_rectangle(pos[0]+side_len, pos[1], pos[0]+2*side_len, pos[1]+side_len, outline=col, fill=col)
    else:
        draw_tree(tree_list[1], [pos[0]+side_len,pos[1]], side_len / 2)
    if isinstance(tree_list[2], int):
        col=rgb_to_hex(tree_list[2], tree_list[2], tree_list[2])
        canvas.create_rectangle(pos[0]+side_len, pos[1]+side_len, pos[0]+2*side_len, pos[1]+2*side_len, outline=col, fill=col)
    else:
        draw_tree(tree_list[2], [pos[0]+side_len,pos[1]+side_len], side_len / 2)
    if isinstance(tree_list[3], int):
        col=rgb_to_hex(tree_list[3], tree_list[3], tree_list[3])
        canvas.create_rectangle(pos[0], pos[1]+side_len, pos[0]+side_len, pos[1]+2*side_len, outline=col, fill=col)
    else:
        draw_tree(tree_list[3], [pos[0],pos[1]+side_len], side_len / 2)

def traverse_tree(tree_list, num=0): #---------- testing
    for element in tree_list:
        if type(element) is list:
            num = traverse_tree(element, num)
        else:
            print(str(num) + ":", element)
            num += 1
    return num

def rebuild_tree_(tree_list): #---------- testing
    new_tree = []
    for element in tree_list:
        if type(element) is list:
            new_tree.append(rebuild_tree_(element))
        else:
            new_tree.append(element)
    return new_tree

def rebuild_tree(tree_list, node_pos=0): #---------- testing
    new_tree = []
    for element in tree_list:
        if type(element) is list:
            branch, node_pos = rebuild_tree(element, node_pos)
            new_tree.append(branch)
        else:
            new_tree.append(element)
            node_pos += 1
    return new_tree, node_pos

def replace_node(tree_list, node_pos, branch, curr_node_pos=0):
    #print("tree_list:", tree_list, "node_pos:", node_pos, "branch:", branch)
    new_tree = []
    for element in tree_list:
        #print("element:", element, "curr_node_pos:", curr_node_pos)
        if type(element) is list:
            tree_part, curr_node_pos = replace_node(element, node_pos, branch, curr_node_pos)
            new_tree.append(tree_part)
        elif node_pos == curr_node_pos:
            new_tree.append(branch)
            curr_node_pos += 1
        else:
            new_tree.append(element)
            curr_node_pos += 1
    return new_tree, curr_node_pos

def create_tree(commands):
    commands_work = commands[:]
    if len(commands_work) < 1:
        return [0,0,0,0]
    tree_list = commands_work.pop(0)[1]
    for command in commands_work:
        #print(command)
        tree_list, pos = replace_node(tree_list, command[0], command[1])
    return tree_list

def copy_commands_(commands):     #--- testing
    new_commands = []
    for command in commands:
        new_commands.append(command)
        cur_pos += 1
    return new_commands

def copy_commands(commands):
    list_len = len(commands)
    cur_pos = 0
    rand1 = random.randint(0, list_len)
    rand2 = random.randint(0, list_len - 1)
    # 0 1 2 3 4 5
    #     ^
    # 0 1.3 4 5
    # 0 1 2 3 4
    if rand2 > rand1 - 1:
        rand2 += 1
    else:
        temp = rand2
        rand2 = rand1
        rand1 = temp
    #print("rand1:", rand1, "rand2:", rand2)
    # x----x---x-------x
    #      1   2
    # x----x   x-------x
    # x----x---x
    #      x---x-------x
    new_commands = []
    part1 = commands[:rand1]
    part2 = commands[rand1:rand2]
    part3 = commands[rand2:]
    if part1 + part3 == []:
        return part2 + part2
    if random.randint(0,1) < 1:
        #print("13")
        return part1 + part3
    else:
        #print("1223")
        return part1 + part2 + part2 + part3

def repair_commands(commands):
    num_nodes = 0
    for command in commands:
        #print(command[0], "num_nodes:", num_nodes, ", ", command[0]%(num_nodes+1))
        command[0] = command[0]%(num_nodes+1)
        num_nodes += 3

def mutate_lightness(val):
    if random.randint(0,100) < 10:
       return (random.randint(0, 256) + 2*val) // 3
    else:
       return val

def mutate_pos(val, num):
    #0,4
    #1,7
    #2,10
    #3,13
    #n*3+1
    if random.randint(0,100) < 10:
       return (random.randint(0, num*3+1))
    else:
       return val

def copy_commands_2(commands):
    list_len = len(commands)
    new_commands = []
    for command in commands:
        new_commands.append([mutate_pos(command[0], list_len), [mutate_lightness(command[1][0]),
                                          mutate_lightness(command[1][1]),
                                          mutate_lightness(command[1][2]),
                                          mutate_lightness(command[1][3])
                                         ]
                            ])
    return new_commands

def key_pressed(event):
    print("pressed", repr(event.char))

def evol_commands(commands):
    if len(commands) < 2:
        new_commands = [commands[0], commands[0]]
        new_commands = copy_commands_2(new_commands)
    else:
        new_commands = copy_commands(commands)
        repair_commands(new_commands)
        new_commands = copy_commands_2(new_commands)
    return new_commands
    
    
def mouse_button_clicked(event):
    global commands_0, commands_1, commands_2, commands_3, commands_4, commands_5, commands_6
    #print("clicked at", event.x, event.y)
    valid_click = False
    if start_x < event.x < start_x + 2*side_len and start_y < event.y < start_y + 2*side_len:
        #print("0")
        #print("rules_0:", rules_0)
        #draw_tree(create_tree(rules_0), [start_x,start_y], side_len)
        valid_click = True
    #---
    if start_x < event.x < start_x + 2*side_len and 2*start_y+2*side_len < event.y < 2*start_y + 4*side_len:
        print("1")
        commands_0 = commands_1
        valid_click = True
    if 2*start_x + 2*side_len < event.x < 2*start_x + 4*side_len and 2*start_y+2*side_len < event.y < 2*start_y + 4*side_len:
        print("2")
        commands_0 = commands_2
        valid_click = True
    if 3*start_x + 4*side_len < event.x < 3*start_x + 6*side_len and 2*start_y+2*side_len < event.y < 2*start_y + 4*side_len:
        print("3")
        commands_0 = commands_3
        valid_click = True
    #---
    if start_x < event.x < start_x + 2*side_len and 3*start_y+4*side_len < event.y < 3*start_y + 6*side_len:
        print("4")
        commands_0 = commands_4
        valid_click = True
    if 2*start_x + 2*side_len < event.x < 2*start_x + 4*side_len and 3*start_y+4*side_len < event.y < 3*start_y + 6*side_len:
        print("5")
        commands_0 = commands_5
        valid_click = True
    if 3*start_x + 4*side_len < event.x < 3*start_x + 6*side_len and 3*start_y+4*side_len < event.y < 3*start_y + 6*side_len:
        print("6")
        commands_0 = commands_6
        valid_click = True
    if valid_click:
        commands_1 = evol_commands(commands_0)
        commands_2 = evol_commands(commands_0)
        commands_3 = evol_commands(commands_0)
        commands_4 = evol_commands(commands_0)
        commands_5 = evol_commands(commands_0)
        commands_6 = evol_commands(commands_0)
        draw_all()
    #print(commands_0)
    print("-------------------------")

def close_window(): 
    window.destroy()

def draw_all():
    print(len(commands_0), len(commands_1), len(commands_2), len(commands_3), len(commands_4), len(commands_5), len(commands_6))
    draw_tree(create_tree(commands_0), [start_x,start_y], side_len)
    #---
    draw_tree(create_tree(commands_1), [start_x,2*start_y+2*side_len], side_len)
    draw_tree(create_tree(commands_2), [2*start_x+2*side_len,2*start_y+2*side_len], side_len)
    draw_tree(create_tree(commands_3), [3*start_x+4*side_len,2*start_y+2*side_len], side_len)
    #---
    draw_tree(create_tree(commands_4), [start_x,3*start_y+4*side_len], side_len)
    draw_tree(create_tree(commands_5), [2*start_x+2*side_len,3*start_y+4*side_len], side_len)
    draw_tree(create_tree(commands_6), [3*start_x+4*side_len,3*start_y+4*side_len], side_len)

side_len = 128
start_x = 10
start_y = 10

window = Tk()

canvas = Canvas(window, width=805, height=805)
window.bind_all("<Key>", key_pressed)
canvas.bind("<Button-1>", mouse_button_clicked)
canvas.pack()    
frame = Frame(window)
frame.pack()
button = Button (frame, text = "quit", command = close_window)
button.pack()

#canvas.create_line(0, 0, 200, 100)
#canvas.create_line(0, 100, 200, 0, fill="red", dash=(4, 4))
#canvas.create_rectangle(25, 25, 50, 50, outline="blue", fill="blue")

#img_tree = [[123,76,45,190],35,[78,23,[23,[70,220,[75,12,210,30],115],12,150],56],72]
img_tree = [0,0,0,0]
"""
                   // 0 - 1
0,[0,35,0,72]      // [0,35,0,72] - 4
0,[123,76,45,190]  // [[123,76,45,190],35,0,72] - 7
6,[78,23,0,56]     // [[123,76,45,190],35,[78,23,0,56],72] - 10
7,[23,0,12,150]    // [[123,76,45,190],35,[78,23,[23,0,12,150],56],72] - 13
8,[70,220,0,115]   // [[123,76,45,190],35,[78,23,[23,[70,220,0,115],12,150],56],72] - 16
10,[75,12,210,30]  // [[123,76,45,190],35,[78,23,[23,[70,220,[75,12,210,30],115],12,150],56],72] - 19

[
[0,[0,35,0,72]],
[0,[123,76,45,190]],
[6,[78,23,0,56]],
[7,[23,0,12,150]],
[8,[70,220,0,115]],
[10,[75,12,210,30]]
]

"""

commands_0 = commands_1 = commands_2 = commands_3 = commands_4 = commands_5 = commands_6 =[[0,[0,0,0,0]]]

draw_all()

#traverse_tree([[123,76,45,190],35,[78,23,[23,[70,220,[75,12,210,30],115],12,150],56],72])

"""
commands = [
[0,[0,35,0,72]],           # [0,35,0,72]
[0,[123,76,45,190]],       # [[123,76,45,190],35,0,72]
[5,[78,23,0,56]],          # [[123,76,45,190],35,[78,23,0,56],72]
[7,[23,0,12,150]],         # [[123,76,45,190],35,[78,23,[23,0,12,150],56],72]
[8,[70,220,0,115]],        # [[123,76,45,190],35,[78,23,[23,[70,220,0,115],12,150],56],72]
[10,[75,12,210,30]]        # [[123,76,45,190],35,[78,23,[23,[70,220,[75,12,210,30],115],12,150],56],72]
]
"""

commands = [
[0,[0,35,0,72]],
[0,[123,76,45,190]],
[5,[78,23,0,56]],
[7,[23,0,12,150]],
[8,[70,220,0,115]],
[10,[75,12,210,30]]
]

print(create_tree(commands))
#print(rebuild_tree([1,2,3,4]))
#print(rebuild_tree([1,[1,2,3,4],3,[1,2,[1,2,3,4],4]]))
print("copy:")
print(copy_commands(commands))
#test_list = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]
test_list = [1]
test_list_2 = [[0,[0,0,0,0]]]

for i in range(0,20):
    #print(copy_commands(test_list))
    #copy_commands(test_list)
    print(copy_commands_2(test_list_2))
    
print("----------------------------------------")
repair_commands(commands)
window.mainloop()

